#Code 7.1  User-defined function to generate PPS surrogates
rm(list=ls(all=TRUE))

#Directories
dir.udf<-as.character("~/Documents/BOLOGNA/Didattica/NLTS/surrogate/UserDefinedFunctions")
dir.data<-as.character("~/Documents/BOLOGNA/Didattica/NLTS/surrogate/data")
dir.results<-as.character("~/Documents/BOLOGNA/Didattica/NLTS/surrogate/results")

#Load User-Defined Functions
setwd(dir.udf)
dump("embed_udf", file="embed_udf.R")
source("embed_udf.R")

#Load Data from an external .csv file
setwd(dir.data)
ts<-read.csv("Mayport.csv")
x<-ts$signal

PPS_udf<-function(x,nsurr,lb,ub){

#Step 1: Embed time series
results.embed_udf<-embed_udf(x)
Mx<-results.embed_udf[[4]] #embedded data matrix

#Step 2: Distances between points on attractor (i.e., row of Mx)
   #Step 2a: Calculate distance matrix from embedded time series matrix 'Mx'
   library(fields)
   dist<-rdist(Mx)

   #Step 2b: Scale each column element to maximum distance within that column in 'dist'
   sf<-apply(dist,2,max) #apply(base) [find maximum distance in each column]
   dist.tr<-t(dist)
   dist.sc<-t(dist.tr/sf)

#Step 3: Random walk on attractor (i.e., over rows of embedded time series matrix 'x.em'.
   surr.PPS<-matrix(0,length(x),nsurr)  #matrix to hold PPS surrogate vectors
   for(j in 1:nsurr){
      surr.vector<-matrix(0,length(x),1)  #matrix to hold PPS vector as it is created
                                          #element by element
      for(i in 1:length(x)){

        #Step 3a: Pick reference point on attractor at random
        rp.1<-sample(nrow(Mx),1)

        #Step 3b: Identify neighboring points
         ngh.lb<-which(lb < dist.sc[,rp.1]) #indices satisfying lower bound 'lb' of nghbhd
         ngh.ub<-which(ub > dist.sc[,rp.1]) #indices satisfying upper bound 'ub' of nghbhd
         int<-intersect(ngh.lb,ngh.ub)    #indices satisfying both bounds in neighborhood

         #Step 3c: Pick point in neighborhood at random
         rp.2<-sample(int,1)
         surr.vector[i,1]<-Mx[,1][rp.2] #1st column of Mx gives element of surrogate vector
      }   #end iloop

     surr.PPS[,j]<-surr.vector
   }  #end jloop

   return(surr.PPS)
}  #end user-defined function

surr.PPS<-PPS_udf(x,nsurr=4,lb=0.01,ub=0.4)
surr.PPS